package com.cs.bd.daemon;

import android.app.Service;
import android.content.Context;
import android.content.Intent;

import com.cs.bd.daemon.def.config.AssistReceiver;
import com.cs.bd.daemon.def.config.AssistService;
import com.cs.bd.daemon.def.config.MainReceiver;
import com.cs.bd.daemon.def.config.MainService;
import com.cs.bd.daemon.receiver.BootCompleteReceiver;
import com.cs.bd.daemon.util.CustomAlarm;
import com.cs.bd.daemon.util.DaemonProcessAlarm;
import com.cs.bd.daemon.util.LogUtils;
import com.cs.bd.daemon.util.MpSwitch;
import com.cs.bd.daemon.util.PackageUtils;
import com.cs.bd.daemon.util.ProcessUtils;

import java.util.List;

import static com.cs.bd.daemon.DaemonConstants.PERMITTING_SWITCH_NAME;

/**
 * @author
 */
public class DaemonClient {
    private static final String TAG = DaemonConstants.LogTag.TAG;
    private static DaemonClient sInstance;

    private Context mContext;
    private DaemonConfigurations mConfigurations;

    public DaemonClient() {
//        //混淆检测
//        boolean isProguard = !getClass().getCanonicalName().endsWith("DaemonClient");
//        if (!isProguard) {
//            throw new IllegalAccessError("未正确混淆守护sdk，客户端必须移除所有守护sdk的keep!!!");
//        }
    }

    public static DaemonClient getInstance() {
        if (sInstance == null) {
            sInstance = new DaemonClient();
        }
        return sInstance;
    }

    public Context getContext() {
        return mContext;
    }

    /**
     * 初始化，使用默认配置
     */
    public void init(Context base) {
        init(createDefaultConfigurations(base));
    }

    /**
     * 建议走 init(Context)
     *
     * @param configurations
     */
    @Deprecated
    public void init(DaemonConfigurations configurations) {
        mConfigurations = configurations;
        if (LogUtils.sIsLog) {
            LogUtils.i(TAG, String.format("DaemonClient::init-->{persistent:%s}, {daemon:%s}",
                    configurations.mPersistentConfig.toString(), configurations.mDaemonAssistantConfig.toString()));
        }
    }

    /**
     * 设置debug模式，打开log
     */
    public void setDebugMode() {
        LogUtils.sIsLog = true;
    }

    /**
     * 获取配置信息
     *
     * @return
     */
    public DaemonConfigurations getConfigurations() {
        return mConfigurations;
    }

    /**
     * 获取被守护的进程服务名称
     *
     * @return
     */
    public String getPersistentServiceName() {
        if (mConfigurations != null) {
            return mConfigurations.getPersistentServiceName();
        }
        return null;
    }

    /**
     * 关联上下文
     *
     * @param base
     */
    public void onAttachBaseContext(Context base) {
        mContext = base;

        initDaemon(base);

        //防止开机广播被杀软禁止
        PackageUtils.setComponentEnable(base, BootCompleteReceiver.class.getName());
    }

    /**
     * do some thing about daemon
     *
     * @param base
     */
    private void initDaemon(Context base) {
        LogUtils.d(TAG, "DaemonClient::initDaemon-->enter");
        if (mConfigurations == null) {
            LogUtils.d(TAG, "DaemonClient::initDaemon-->exit, mConfigurations == null");
            return;
        }
        if (!isDaemonPermitting(base)) {
            LogUtils.d(TAG, "DaemonClient::initDaemon-->exit, isDaemonPermitting == false");
            return;
        }
//        if (DaemonConstants.isNotSupportDaemon()) {
//            LogUtils.d(TAG, "DaemonClient::initDaemon-->exit, isNotSupportDaemon == true, fingerprint:" + Build.FINGERPRINT);
//            return;
//        }

        String processName = ProcessUtils.getProcessName(mContext);
        String packageName = base.getPackageName();
        if (LogUtils.sIsLog) {
            LogUtils.d(TAG, "DaemonClient::initDaemon-->processName:" + processName + ", pkgName:" + packageName);
        }

        IDaemonStrategy strategy = IDaemonStrategy.Fetcher.fetchStrategy(base, mConfigurations);
        strategy.setProcessName(processName);
        try {
            //这里都会crash...
            if (processName.equals(mConfigurations.mPersistentConfig.mProcessName)) {
                strategy.onPersistentCreate(base, mConfigurations);
                daemonOtherPersistentProcesses(base);
            } else if (processName.equals(mConfigurations.mDaemonAssistantConfig.mProcessName)) {
                strategy.onDaemonAssistantCreate(base, mConfigurations);
                daemonPersistentProcesse(base);
            } else {
                daemonPersistentProcesse(base);
                strategy.onInitialization(base);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        //todo debug by matt
//        ProcessUtils.releaseIO();

        //初始化Acti监听
//        MpActivityMonitor.getInstance(mContext);

        doStatisticsDaemonEffect();
    }

    private void daemonPersistentProcesse(Context context) {
        if (LogUtils.sIsLog) {
            LogUtils.i(TAG, "[DaemonClient#daemonPersistentProcesse] ");
        }
        PackageUtils.startServiceIfNotRunning(mContext, mConfigurations.getPersistentServiceName());
    }

    /**
     * 守护其它常驻进程
     */
    private void daemonOtherPersistentProcesses(Context context) {
        if (LogUtils.sIsLog) {
            LogUtils.i(TAG, "[DaemonClient#daemonOtherPersistentProcesses] ");
        }

        int alarmId = DaemonProcessAlarm.ALARMID_ACTIVATE_OTHRE_PERSISTENT_PROCESS;
        long triggerDelay = mConfigurations.getDaemonOtherPersistentProcessTrigger() * 1000;
        long interval = mConfigurations.getDaemonOtherPersistentProcessInterval() * 1000;
        DaemonProcessAlarm.getAlarm(context).alarmRepeat(alarmId, triggerDelay, interval, true, new CustomAlarm.OnAlarmListener() {

            @Override
            public void onAlarm(int alarmId) {
                //拉起辅助进程
                PackageUtils.startServiceIfNotRunning(mContext, mConfigurations.getAssistantServiceName());

                //拉起其它常驻进程
                List<String> otherPersistentServices = mConfigurations.getOtherPersistenServices();
                if (otherPersistentServices == null || otherPersistentServices.size() <= 0) {
                    return;
                }
                for (String serviceName : otherPersistentServices) {
                    PackageUtils.startServiceIfNotRunning(mContext, serviceName);
                }
            }
        });

    }

    /**
     * 是否开启守护功能
     *
     * @param context
     * @return
     */
    public boolean isDaemonPermitting(Context context) {
        return new MpSwitch(mContext, PERMITTING_SWITCH_NAME, true).getValue();
    }

    /**
     * 设置是否开启守护功能
     *
     * @param context
     * @param isPermitting 是否开启，false时若守护已启动则关停。true时要下次{@link #onAttachBaseContext}被调用才生效
     */
    public void setDaemonPermiiting(Context context, boolean isPermitting) {
        LogUtils.i(TAG, "[DaemonClient#setDaemonPermiiting] isPermitting: " + isPermitting);
        new MpSwitch(mContext, PERMITTING_SWITCH_NAME, true).setValue(isPermitting);
    }

    /**
     * 设置forgroundService为前台服务，在{@link Service#onCreate()}里调用
     *
     * @param forgroundService
     * @param innerServiceClass
     * @备注 forgroundService和innerServiceClass必须配置在同一个进程内
     * @备注 对gp包，开了守护的应用，不建议再设置前台进程，会影响用户升级
     * @备注 仅对系统版本<= 2 4 的可以隐藏通知栏 ， 高版本建议走常驻通知栏来设置前台服务
     */
    public void setForgroundService(Service forgroundService, Class<? extends NotificationAssistService> innerServiceClass) {
        NotificationAssistService.setForgroundService(forgroundService, innerServiceClass);
    }

    /**
     * 统计守护效果, 在{@link Service#onStartCommand(Intent, int, int)}和{@link android.content.BroadcastReceiver#onReceive(Context, Intent)}里调用
     */
    private void doStatisticsDaemonEffect() {
        if (mConfigurations == null
                || !mConfigurations.isStatisticsDaemonEffect()) {
            //参数异常、不开启统计
            return;
        }
        //todo debug


    }

    /**
     * 构建守护配置
     *
     * @return
     */
    private static DaemonConfigurations createDefaultConfigurations(Context context) {
        //构建主常驻进程配置信息
        DaemonConfigurations.DaemonConfiguration configuration1 = new DaemonConfigurations.DaemonConfiguration(
                context.getPackageName(),
                MainService.class.getCanonicalName(),
                MainReceiver.class.getCanonicalName());
        //构建辅助进程配置信息
        DaemonConfigurations.DaemonConfiguration configuration2 = new DaemonConfigurations.DaemonConfiguration(
                context.getString(R.string.csd_assist_process_name),
                AssistService.class.getCanonicalName(),
                AssistReceiver.class.getCanonicalName());
        //构建配置信息
        DaemonConfigurations configs = new DaemonConfigurations(configuration1, configuration2);

        return configs;
    }
}
