package com.cs.bd.daemon;

import android.content.Context;

import com.cs.bd.daemon.statistics.Process103Statistic.IStatistic103;

import java.util.ArrayList;
import java.util.List;

/**
 * the configurations of Daemon SDK, contains two process configuration.
 * @author
 *
 */
public class DaemonConfigurations {

	/**
	 * 主常驻进程配置
	 */
	public final DaemonConfiguration mPersistentConfig;
	/**
	 * 辅助进程配置
	 */
	public final DaemonConfiguration mDaemonAssistantConfig;
	public final DaemonListener mListener;
	/**
	 * daemon监控时间间隔，单位s
	 */
	private int mDaemonWatchInterval = DaemonConstants.DAEMON_WATCH_INTERVAL;
	/**
	 * daemon监控时间间隔(android版本21以上)，单位s
	 */
	private int mDaemonWatchIntervalAboveAPI21 = DaemonConstants.DAEMON_WATCH_INTERVAL_ABOVE_API21;
	/**
	 * 多长时间后，第一次唤醒其它常驻进程，单位s
	 */
	private int mDaemonOtherPersistentProcessTrigger = DaemonConstants.DAEMON_OTHER_PERSISTENT_PROCESS_TRIGGER;
	/**
	 * 唤醒其它常驻进程的监控时间间隔，单位s
	 */
	private int mDaemonOtherPersistentProcessInterval = DaemonConstants.DAEMON_OTHER_PERSISTENT_PROCESS_INTERVAL;
	/**
	 * 除了{@link #mPersistentConfig}外，其它常驻进程的服务名称
	 */
	private List<String> mOtherPersistenServices;
	/**
	 * 是否统计守护效果
	 */
	private boolean mIsStatisticsDaemonEffect = false;
	/**
	 * 崩溃保护配置
	 */
	private CrashConfiguration mCrashConfig = CrashConfiguration.defaultConfig();
	/**
	 * 是否gp包
	 */
	private boolean mIsGp = true;
	/**
	 * 是否统计应用存活时长
	 */
	private boolean isStatisticsLifeTime = false;
	/**
	 * 统计接口，由客户端实现
	 */
	private IStatistic103 mStatistic103;

	public DaemonConfigurations(DaemonConfiguration persistentConfig, DaemonConfiguration daemonAssistantConfig) {
		this.mPersistentConfig = persistentConfig;
		this.mDaemonAssistantConfig = daemonAssistantConfig;
		this.mListener = null;
	}
	
	public DaemonConfigurations(DaemonConfiguration persistentConfig, DaemonConfiguration daemonAssistantConfig, DaemonListener listener) {
		this.mPersistentConfig = persistentConfig;
		this.mDaemonAssistantConfig = daemonAssistantConfig;
		this.mListener = listener;
	}

	public String getPersistentServiceName() {
		if (mPersistentConfig != null) {
			return mPersistentConfig.mServiceName;
		}
		return null;
	}

	public String getAssistantServiceName() {
		if (mDaemonAssistantConfig != null) {
			return mDaemonAssistantConfig.mServiceName;
		}
		return null;
	}

	public int getDaemonWatchInterval() {
		return mDaemonWatchInterval;
	}

	/**
	 * 设置daemon监控时间间隔，单位s。默认60s
	 * @param interval
	 */
	public DaemonConfigurations setDaemonWatchInterval(int interval) {
		if (interval <= 0) {
			interval = 1;
		}
		mDaemonWatchInterval = interval;
		return this;
	}

	public int getDaemonWatchIntervalAboveAPI21() {
		return mDaemonWatchIntervalAboveAPI21;
	}

	/**
	 * 设置daemon监控时间间隔(android版本21以上)，单位s。默认120s
	 * @param interval
	 */
	public DaemonConfigurations setDaemonWatchIntervalAboveAPI21(int interval) {
		if (interval <= 0) {
			interval = 1;
		}
		mDaemonWatchIntervalAboveAPI21 = interval;
		return this;
	}

	public int getDaemonOtherPersistentProcessTrigger() {
		return mDaemonOtherPersistentProcessTrigger;
	}

	/**
	 * 设置多长时间后，第一次唤醒其它常驻进程，单位s
	 * @param triggerDelay
     */
	public DaemonConfigurations setDaemonOtherPersistentProcessTrigger(int triggerDelay) {
		if (triggerDelay <= 0) {
			triggerDelay = 1;
		}
		mDaemonOtherPersistentProcessTrigger = triggerDelay;
		return this;
	}

	public int getDaemonOtherPersistentProcessInterval() {
		return mDaemonOtherPersistentProcessInterval;
	}

	/**
	 * 设置唤醒其它常驻进程的监控时间间隔，单位s
	 * @param interval
     */
	public DaemonConfigurations setDaemonOtherPersistentProcessInterval(int interval) {
		if (interval <= 0) {
			interval = 1;
		}
		mDaemonOtherPersistentProcessInterval = interval;
		return this;
	}

	/**
	 * 添加其它常驻进程配置
	 * @备注 此接口不支持多线程安全，只应在主线程调用
	 */
	public DaemonConfigurations addPersistentService(String serviceName) {
		if (serviceName == null) {
			return this;
		}
		if (mOtherPersistenServices == null) {
			mOtherPersistenServices = new ArrayList<String>();
		}
		mOtherPersistenServices.add(serviceName);
		return this;
	}

	public List<String> getOtherPersistenServices() {
		return mOtherPersistenServices;
	}

	/**
	 * 是否统计守护效果
	 * @return
     */
	public boolean isStatisticsDaemonEffect() {
		return mIsStatisticsDaemonEffect && mStatistic103 != null;
	}

	/**
	 * 设置是否统计守护效果
	 * @param statisticsDaemonEffect
     */
	public DaemonConfigurations setStatisticsDaemonEffect(boolean statisticsDaemonEffect) {
		mIsStatisticsDaemonEffect = statisticsDaemonEffect;
		return this;
	}

	/**
	 * 设置保护配置，用来防止因程序BUG导致的频繁重启
	 * @param config
     */
	public DaemonConfigurations setCrashConfig(CrashConfiguration config) {
		mCrashConfig = config;
		return this;
	}

	/**
	 * 是否CN包。gp和国内的优化策略不同
	 * @return
	 */
	public boolean isCN() {
		return !mIsGp;
	}

	/**
	 * 是否gp包。gp和国内的优化策略不同
	 * @param isGp
	 * @return
	 */
    public DaemonConfigurations setIsGp(boolean isGp) {
		mIsGp = isGp;
		return this;
	}

    /**
     * 是否统计进程存活时长
     * @return
     */
	public boolean isStatisticsLifeTime() {
		return isStatisticsLifeTime && mStatistic103 != null;
	}

    /**
     * 是否统计进程存活时长
     * @param value
     * @return
     */
	public DaemonConfigurations setIsStatisticsLifeTime(boolean value) {
		isStatisticsLifeTime = value;
		return this;
	}

    public IStatistic103 getStatistic103() {
        return mStatistic103;
    }

    public void setStatistic103(IStatistic103 statistic103) {
        mStatistic103 = statistic103;
    }

    /**
	 * the configuration of a daemon process, contains process name, service name and receiver name if Android 6.0
	 * @author matt
	 *
	 */
	public static class DaemonConfiguration {
		
		public final String mProcessName;
		public final String mServiceName;
		public final String mReceiverName;
		
		public DaemonConfiguration(String processName, String serviceName, String receiverName) {
			this.mProcessName = processName;
			this.mServiceName = serviceName;
			this.mReceiverName = receiverName;
		}

		@Override
		public String toString() {
			return String.format("[DaemonConfiguration] mProcessName:%s, mServiceName:%s, mReceiverName:%s", mProcessName, mServiceName, mReceiverName);
		}
	}

	/**
	 * 崩溃保护配置，用来防止因程序BUG导致的频繁重启
	 * 策略1， 在一定时间重启超过一定次数，则不再重启
	 */
	public static class CrashConfiguration {
		/**
		 * 观察周期
		 */
		public final long mObserveDuration;
		/**
		 * 重启次数
		 */
		public final int mMaxDeadCount;

		private CrashConfiguration(int maxDeadCount, long inDuration) {
			mMaxDeadCount = maxDeadCount;
			mObserveDuration = inDuration;
		}

		public static CrashConfiguration build(int maxDeadCount, long inDuration) {
			if (inDuration <= 0) {
				return null;
			}
			return new CrashConfiguration(maxDeadCount, inDuration);
		}

		public static CrashConfiguration defaultConfig() {
			return build(3, 15);
		}
	}

	/**
	 * listener of daemon for external
	 * 
	 * @author
	 *
	 */
	public interface DaemonListener {
		void onPersistentStart(Context context);
		void onDaemonAssistantStart(Context context);
		void onWatchDaemonDaed();
	}
}
